<?php
namespace Teknisa\MAN\Services;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Teknisa\Libs\Util\Environment;
use \Exception as Exception;

class MetaService {

    protected $entityManager;
    protected $paramsDelete = array();
    protected $PrevisaoEstoqueService;
    
    public function __construct(\Doctrine\ORM\EntityManager $entityManager, Environment $environment, $PrevisaoEstoqueService)  {
        $this->entityManager = $entityManager;
        $this->environment  = $environment;
        $this->PrevisaoEstoqueService = $PrevisaoEstoqueService;
    }
    
    public function recalculoMetaTodasFiliais ($cdoperador, $nrorg) {
        $params = array( 
			':NRORG'    		=> $nrorg,
			':CDOPERADOR'       => $cdoperador
		);
		
		$filiais = $this->entityManager->getConnection()->fetchAll(self::GET_FILIAIS, $params);

		foreach($filiais as $filial) {
		    $this->recalculoMetaFilial ($filial['CDFILIAL'], $nrorg, $cdoperador, true);
		}
    }  

    public function recalculoMetaFilial ($cdfilial, $nrorg, $cdoperador, $all = false) {
        $result = false;
        //busca o tipo de meta 
        //busca parametro para determinar Procedure
        $getTipoMeta = $this->getTipoMeta($cdfilial);
        
        if($getTipoMeta['IDTIPOMETA'] == 'S'){
            
            //busca dados do historico de meta 
            $params[':CDFILIAL'] = $cdfilial;
            $params[':IDSTATUS'] = 'P';
            
            $historico = $this->entityManager->getConnection()->fetchAssoc(self::GET_DADOS_HISTORICO_META, $params);
            
            if(!$all){
                if(empty($historico)){
                    throw new Exception('Não a meta para a filial/unidade '.$cdfilial.'. Necessário realiazar meta primeiro!');
                }
            }
            
            
            if(!empty($historico)){
                try {
                    $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CALCULA_METAS_RM_SAZONALIDADE');
                    $procedure->addParam(new Param('V_CDFILIAL'  , Param::PARAM_INPUT , $cdfilial, Param::PARAM_TYPE_STR));
                    $procedure->addParam(new Param('V_NRORG', Param::PARAM_INPUT, $nrorg  , Param::PARAM_TYPE_STR));
                    $procedure->addParam(new Param('V_PERCACRES', Param::PARAM_INPUT, $historico['VRPERCACRES'] , Param::PARAM_TYPE_STR));
                    $procedure->addParam(new Param('V_PERIODO', Param::PARAM_INPUT, 'S'  , Param::PARAM_TYPE_STR));
                    $procedure->addParam(new Param('V_DTINI', Param::PARAM_INPUT, substr($historico['DTINICIO'], 0, 10), Param::PARAM_TYPE_STR));
                    $procedure->addParam(new Param('V_DTFIN', Param::PARAM_INPUT, substr($historico['DTFINAL'], 0, 10)  , Param::PARAM_TYPE_STR));
                    $procedure->addParam(new Param('V_CDANO', Param::PARAM_INPUT, $historico['CDANO']  , Param::PARAM_TYPE_INT));
                    $procedure->addParam(new Param('V_CDOPERADOR', Param::PARAM_INPUT, $cdoperador  , Param::PARAM_TYPE_STR));
                    $procedure->addParam(new Param('V_CDMETAPREV', Param::PARAM_INPUT, $historico['CDMETAPREV']  , Param::PARAM_TYPE_STR));
                    $procedure->addParam(new Param('V_MESINICIO', Param::PARAM_INPUT, $historico['CDMESINICIAL']  , Param::PARAM_TYPE_INT));
                    $procedure->addParam(new Param('V_MESFINAL', Param::PARAM_INPUT, $historico['CDMESFINAL']  , Param::PARAM_TYPE_INT));
                    $procedure->execute();   
                    
                    $result = true;
                } catch (\Exception $e) {
                    throw $e;
                }    
            }else{
                 $result = true;;
            }
            
            return $result;
            
         }else{
             
             try {
                $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CALCULA_METAS_RM');
                $procedure->addParam(new Param('V_CDFILIAL'  , Param::PARAM_INPUT , $cdfilial, Param::PARAM_TYPE_STR));
                $procedure->addParam(new Param('V_NRORG', Param::PARAM_INPUT, $nrorg  , Param::PARAM_TYPE_INT));
                $procedure->execute();   
                
                $result = true;
            } catch (\Exception $e) {
                throw $e;
            }
             
            return $result;
             
         }
        
    }
    
    public function getTipoMeta ($cdfilial) {
        $params = array( 
			':CDFILIAL'    		=> $cdfilial
		);
		
		try {
		    $tipoMeta = $this->entityManager->getConnection()->fetchAssoc(self::GET_PARAM_TIPOMETA, $params);
		} catch (\Exception $e) {
            $tipoMeta['IDTIPOMETA'] = 'N';
        }
		return $tipoMeta;
    }  
    
    public function estimaMetaFilial ($cdfilial, $nrorg, $percentual, $periodo, $dtini, $dtfin, $dtano, $mesInicio, $mesFinal) {
        $result = false;
        $tipoMeta = 'ESTIMA_META';
        $cdoperador = $this->environment->getCdOperador();
        $params["VENDASDIARIAS"]["start"] = $dtini;
        $params["VENDASDIARIAS"]["end"] = $dtfin;
        $params["NRORG"] = $nrorg;
        $params["CDFILIAL"] = $cdfilial;
        
        //busca parametro para determinar Procedure
        $getTipoMeta = $this->getTipoMeta($cdfilial);
        
        if($getTipoMeta['IDTIPOMETA'] == 'S'){
            $tipoMeta = 'ESTIMA_META_SAZONALIDADE';
            
            $verificaParam = $this->verificaParam($params);
            if(!$verificaParam['status']){ 
                throw new Exception($verificaParam['message']);
            }
            
            if(empty($mesInicio)){
               $mesInicio = 1; 
            }
            
            if(empty($mesFinal)){
               $mesFinal = 12; 
            }
            
            $params["CDMESINICIAL"] = $mesInicio;
            $params["CDMESFINAL"] = $mesFinal;
            $params["CDANO"] = $dtano;
            $params["IDSTATUS"] = 'C';
            
            //verifica se ja nao foi realizada 
            $getHistoricoParam = $this->getHistoricoParam($params);
            if(!$verificaParam['status']){ 
                throw new Exception($verificaParam['message']);
            }
            
            
        }
        
     try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), $tipoMeta);
            $procedure->addParam(new Param('V_CDFILIAL'  , Param::PARAM_INPUT , $cdfilial, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('V_NRORG', Param::PARAM_INPUT, $nrorg, Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('V_PERCACRES', Param::PARAM_INPUT, $percentual));
            $procedure->addParam(new Param('V_PERIODO', Param::PARAM_INPUT, $periodo));
            $procedure->addParam(new Param('V_DTINI', Param::PARAM_INPUT, $dtini));
            $procedure->addParam(new Param('V_DTFIN', Param::PARAM_INPUT, $dtfin));
            $procedure->addParam(new Param('P_CDANO', Param::PARAM_INPUT, $dtano, Param::PARAM_TYPE_INT));
            
            if($tipoMeta == 'ESTIMA_META_SAZONALIDADE'){
                $procedure->addParam(new Param('V_CDOPERADOR', Param::PARAM_INPUT, $cdoperador, Param::PARAM_TYPE_STR)); 
                
                if(empty($mesInicio)){
                   $mesInicio = 1; 
                }
                
                if(empty($mesFinal)){
                   $mesFinal = 12; 
                }
                
                $procedure->addParam(new Param('V_MESINICIO', Param::PARAM_INPUT, $mesInicio, Param::PARAM_TYPE_STR));   
                $procedure->addParam(new Param('V_MESFINAL', Param::PARAM_INPUT, $mesFinal, Param::PARAM_TYPE_STR));   
                
            }
            
            $procedure->execute();   
            
            $result = true;
        } catch (\Exception $e) {
            throw $e;
        }
        
        return $result;
    }
    
    public function verificaParam($rows){
        $return = [
            "status" => true,
            "message" => ''
        ];
        
        $diasComAbertura = $this->PrevisaoEstoqueService->getDiasComAbertura($rows);
        $arrayDiasVendidos = self::getDiasVendidos($rows['VENDASDIARIAS']['start'], $rows['VENDASDIARIAS']['end']);
        
        if(count($diasComAbertura) != count($arrayDiasVendidos)){ //verifica se existe dias sem abertura de caixa
            $diff = array_diff($arrayDiasVendidos, $diasComAbertura);
            if(abs(count($arrayDiasVendidos) - count($diff)) < 21 ){
                $return['status'] = false;
                $return['message'] = "O período possui poucos dias com vendas.";
            } 
        } 
        
        if(count($arrayDiasVendidos) < 21){
            $return['status'] = false;
            $return['message'] = "O período possui poucos dias com vendas, mínimo 21 dias";
        }
        
        return $return;
    }
    
    public function getHistoricoParam($rows){
        $return = [
            "status" => true,
            "message" => ''
        ];
        
        $params[':NRORG'] = $rows['NRORG'];
        $params[':CDFILIAL'] = $rows['CDFILIAL'];
        $params[':CDMESINICIAL'] = $rows['CDMESINICIAL'];
        $params[':CDMESFINAL'] = $rows['CDMESFINAL'];
        $params[':IDSTATUS'] = $rows['IDSTATUS'];
        $params[':CDANO'] = $rows['CDANO'];
        
        
        $historico = $this->entityManager->getConnection()->fetchAll(self::GET_HISTORICO_META, $params);
        
        if(empty($historico)){
            $return['status'] = false;
            $return['message'] = "Há uma meta com os mesmos dados realizada!";
        }
        
        return $return;
    }
    
    
    
    
    private function getDiasVendidos($dtInicial, $dtFinal){
        $dateStart 		= self::returnDataYMD($dtInicial);
        $dateEnd 		= self::returnDataYMD($dtFinal);
        $dateRange = array();
        for($qtDia = 0; $dateStart <= $dateEnd; $qtDia++){
            $dateRange[] = $dateStart->format('d/m/Y');
            $dateStart = $dateStart->modify('+1day');
        }
        return $dateRange;
    }
    
    private function returnDataYMD ($date){
         return new \DateTime(date("Y-m-d", strtotime(str_replace("/", "-", $date))));
    }
    
    public function syncDashboardData () {
        
        try{
            $this->entityManager->getConnection()->executeQuery(self::UPDATE_VENDAS_DIA);
    		$this->entityManager->getConnection()->executeQuery(self::LIMPAR_TABELA_VENDADIAMODALIDADE);
    		$this->entityManager->getConnection()->executeQuery(self::LIMPAR_TABELA_VENDADIATOTAL);
    		$this->entityManager->getConnection()->executeQuery(self::LIMPAR_TABELA_VENDAHORATOTAL);
    		$procedure = new StoredProcedure($this->entityManager->getConnection(), 'REPLICA_VENDAS_RM');
            $retorno = $procedure->execute();
            return $retorno;
        }catch (\Exception $e) {
            return $e->getMessage();
        }
	}  
    
    private function setParamsDeleteFilhos($conditions){
        return $this->paramsDelete = array(
            ':CDFILIAL' => $conditions['CDFILIAL'],
            ':CDANO' => $conditions['CDANO'],
            ':CDMES' => $conditions['CDMES'],
            ':NRORG' => $conditions['NRORG']
        );
    }
    
    public function deleteRegistrosFilhos ($conditions){
        try{
            $this->setParamsDeleteFilhos($conditions);
            //DELETAR METAMODALIDADE
            $this->entityManager->getConnection()->executeQuery(self::DELETE_METAMODALIDADE, $this->paramsDelete);
            //DELETAR NETAHORA
            $this->entityManager->getConnection()->executeQuery(self::DELETE_METAHORA, $this->paramsDelete);
            //DELETAR METADIA
            $this->entityManager->getConnection()->executeQuery(self::DELETE_METADIA, $this->paramsDelete);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    const GET_PARAM_TIPOMETA = "SELECT IDTIPOMETA FROM PARAMFILIAL WHERE CDFILIAL = :CDFILIAL AND ROWNUM = 1";
    
    const GET_FILIAIS = "SELECT CDFILIAL FROM FILIOPER WHERE CDOPERADOR = :CDOPERADOR AND NRORG = :NRORG";
    
    const GET_HISTORICO_META = "SELECT * FROM METAPREVISAOHIST WHERE CDFILIAL = :CDFILIAL AND CDANO = :CDANO AND CDMESINICIAL = :CDMESINICIAL AND CDMESFINAL = :CDMESFINAL AND IDSTATUS <> :IDSTATUS ";
    
    const GET_DADOS_HISTORICO_META = "SELECT * FROM METAPREVISAOHIST WHERE CDFILIAL = :CDFILIAL AND IDSTATUS = :IDSTATUS ";
    
    const UPDATE_VENDAS_DIA = "
		UPDATE VENDA
			SET IDCONTVEND = 'N'
		WHERE TO_DATE(TO_CHAR(DTVENDA, 'DD/MM/YYYY'))  = TO_DATE(TO_CHAR(sysdate, 'DD/MM/YYYY'))
	";
	
	const LIMPAR_TABELA_VENDADIAMODALIDADE = "
		DELETE FROM VENDADIAMODALIDADE
        WHERE TO_DATE(TO_CHAR(DTVENDA, 'DD/MM/YYYY'))  = TO_DATE(TO_CHAR(sysdate, 'DD/MM/YYYY'))
	";
	
	const LIMPAR_TABELA_VENDADIATOTAL = "
		DELETE FROM VENDADIATOTAL
        WHERE TO_DATE(TO_CHAR(DTVENDA, 'DD/MM/YYYY'))  = TO_DATE(TO_CHAR(sysdate, 'DD/MM/YYYY'))
	";
	
	const LIMPAR_TABELA_VENDAHORATOTAL = "
		DELETE FROM VENDAHORATOTAL
        WHERE TO_DATE(TO_CHAR(DTVENDA, 'DD/MM/YYYY'))  = TO_DATE(TO_CHAR(sysdate, 'DD/MM/YYYY'))
	";
	
	const DELETE_METAMODALIDADE = "
		DELETE FROM METAMODALIDADEDIA
        WHERE CDFILIAL = :CDFILIAL AND CDMES = :CDMES AND CDANO = :CDANO AND NRORG = :NRORG
	";
	
	const DELETE_METAHORA = "
		DELETE FROM METAHORA
        WHERE CDFILIAL = :CDFILIAL AND CDMES = :CDMES AND CDANO = :CDANO AND NRORG = :NRORG
	";
    
    const DELETE_METADIA = "
		DELETE FROM METADIA
        WHERE CDFILIAL = :CDFILIAL AND CDMES = :CDMES AND CDANO = :CDANO AND NRORG = :NRORG
	";
    
} 